require "import"
import "java.text.SimpleDateFormat"
import "java.util.Date"
import "java.util.Calendar"
import "android.content.Context"
import "android.content.Intent"
import "android.content.IntentFilter"
import "android.os.BatteryManager"
import "java.lang.System"
import "android.net.Uri"
import "com.androlua.*"

-- Function to fetch the current temperature from the AccuWeather website
function fetchTemperature(callback)
    local url = "https://www.accuweather.com/"
    Http.get(url, function(status, data)
        if status == 200 then
            local temp = string.match(data, "%d+°")
            callback(temp)
        else
            callback(nil)
        end
    end)
end

-- Function to get a greeting based on the current time
function getGreeting()
    local hour = tonumber(os.date("%H"))
    if hour < 12 then
        return "Good morning"
    elseif hour < 18 then
        return "Good afternoon"
    elseif hour < 20 then
        return "Good evening"
    else
        return "Good night"
    end
end

-- Function to format dates
function getFormattedDate(calendar)
    local dateFormatter = SimpleDateFormat("EEEE, MMMM dd, yyyy")
    return dateFormatter.format(calendar.getTime())
end

-- Functions to get specific dates
function getFirstDayOfNextMonth(calendar)
    calendar.add(Calendar.MONTH, 1)
    calendar.set(Calendar.DATE, 1)
    return getFormattedDate(calendar)
end

function getLastDayOfNextMonth(calendar)
    calendar.add(Calendar.MONTH, 1)
    calendar.set(Calendar.DATE, calendar.getActualMaximum(Calendar.DATE))
    return getFormattedDate(calendar)
end

function getEndOfCurrentWeek(calendar)
    calendar.set(Calendar.DAY_OF_WEEK, calendar.getFirstDayOfWeek() + 6)
    return getFormattedDate(calendar)
end

function getLastDayOfMonth(calendar)
    calendar.set(Calendar.DATE, calendar.getActualMaximum(Calendar.DATE))
    return getFormattedDate(calendar)
end

-- Main function to gather and speak date, time, battery, and temperature information
function speakDateTimeAndTemperature(context)
    local currentDate = os.date("%A, %B %d, %Y")

    -- Get the current time in milliseconds since epoch
    local currentTimeInMillis = System.currentTimeMillis()

    -- Create a new Date object with the current time in milliseconds
    local date = Date(currentTimeInMillis)
    
    -- Format the time including milliseconds
    local timeFormatter = SimpleDateFormat("hh:mm:ss.SSS a")
    local formattedTime = timeFormatter.format(date)

    local batteryIntent = context.registerReceiver(nil, IntentFilter(Intent.ACTION_BATTERY_CHANGED))
    local level = batteryIntent.getIntExtra(BatteryManager.EXTRA_LEVEL, -1)
    local scale = batteryIntent.getIntExtra(BatteryManager.EXTRA_SCALE, -1)
    local batteryPercent = math.floor((level / scale) * 100)

    -- Get greeting based on the time of day
    local greeting = getGreeting()

    -- Get tomorrow's date
    local calendar = Calendar.getInstance()
    calendar.add(Calendar.DATE, 1)
    local tomorrowDate = getFormattedDate(calendar)
    
    -- Get day after tomorrow's date
    calendar.add(Calendar.DATE, 1)
    local dayAfterTomorrowDate = getFormattedDate(calendar)

    -- Get end of the month date
    calendar = Calendar.getInstance()
    local endOfMonthDate = getLastDayOfMonth(calendar)
    
    -- Get first and last day of next month
    local firstDayOfNextMonth = getFirstDayOfNextMonth(calendar)
    calendar = Calendar.getInstance()
    local lastDayOfNextMonth = getLastDayOfNextMonth(calendar)
    
    -- Get end of current week date
    calendar = Calendar.getInstance()
    local endOfWeekDate = getEndOfCurrentWeek(calendar)
    
    fetchTemperature(function(temp)
        local temperatureMessage = temp and ("The current temperature is " .. temp) or "Temperature information is not available due to network issues."

        -- Speak the message
        local message = string.format("%s! Today is %s. The time is %s. The battery level is %d percent. %s Tomorrow's date is %s. The day after tomorrow's date is %s. The end of this month is on %s. The first day of next month is %s. The last day of next month is %s. The end of this week is on %s.", 
                                      greeting, currentDate, formattedTime, batteryPercent, temperatureMessage, tomorrowDate, dayAfterTomorrowDate, endOfMonthDate, firstDayOfNextMonth, lastDayOfNextMonth, endOfWeekDate)
        context.speak(message)
        service.playSoundTick()
    end)
end

speakDateTimeAndTemperature(this)

return true